#' Load a VCF object
#'
#' Load a \linkS4class{VCF} object from its staged file contents.
#'
#' @param vcf.info Named list of metadata describing a \linkS4class{VCF} object.
#' @param project Any argument accepted by the acquisition functions, see \code{?\link{acquireFile}}.
#' By default, this should be a string containing the path to a staging directory.
#'
#' @return A \linkS4class{VCF} object, expanded or collapsed depending on the specification in \code{vcf.info}.
#'
#' @details
#' This function assumes that the files were generated by the \code{\link{stageObject}} method for \linkS4class{VCF} subclasses.
#' It does \emph{not} load a VCF file!
#' Use \code{\link{scanVcf}} instead if you want to create a \linkS4class{VCF} object from a VCF file.
#'
#' @author Aaron Lun
#'
#' @examples
#' fl <- system.file("extdata", "structural.vcf", package="VariantAnnotation")
#' vcf <- readVcf(fl, genome="hg19")
#'
#' tmp <- tempfile()
#' dir.create(tmp)
#' info <- stageObject(vcf, dir=tmp, path="experiment-1")
#' loadVCF(info, tmp)
#' 
#' @export
#' @import alabaster.base methods
#' @importFrom alabaster.se loadSummarizedExperiment
loadVCF <- function(vcf.info, project) {
    out <- loadSummarizedExperiment(vcf.info, project)

    fixed.meta <- acquireMetadata(project, vcf.info$vcf_experiment$fixed$resource$path)
    fixed.df <- .loadObject(fixed.meta, project)

    info.meta <- acquireMetadata(project, vcf.info$vcf_experiment$info$resource$path)
    info.df <- .loadObject(info.meta, project)

    header.meta <- acquireMetadata(project, vcf.info$vcf_experiment$header$resource$path)
    metadata(out)$header <- .loadObject(header.meta, project)

    if (!is(out, "RangedSummarizedExperiment")) {
        out <- as(out, "RangedSummarizedExperiment")
    }

    if (vcf.info$vcf_experiment$expanded) {
        new("ExpandedVCF", out, fixed=fixed.df, info=info.df)
    } else {
        new("CollapsedVCF", out, fixed=fixed.df, info=info.df)
    }
}
